#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "common.h"
#include "macros.h"
#include "failures.h"
#include "vm.h"
#include "vmlibcall.h"
#include "vmlibcalln.h"

static void int2float(struct vm *machine)
{
	int32_t integer;
	
	return_if_fail(machine != NULL);
	
	integer = vm_stack_get_int(machine);
	vm_stack_poke_float(machine, (float)integer);
}

static void float2int(struct vm *machine)
{
	float fl;
	
	return_if_fail(machine != NULL);
	
	fl = vm_stack_get_float(machine);
	vm_stack_poke_int(machine, (int32_t)fl);
}

static void str2float(struct vm *machine)
{
	char str[STRING_LEN];
	float fl;

	vm_stack_get_string(machine, str);
	fl = strtof(str, NULL);
	vm_stack_poke_float(machine, fl);
}

static void str2int(struct vm *machine)
{
	char str[STRING_LEN];
	int32_t integer;

	vm_stack_get_string(machine, str);
	integer = strtol(str, NULL, 0);
	vm_stack_poke_int(machine, integer);
}

static struct vm_libcall libcalls[] = {
	{ .name = "int2float",	.num = VM_INT2FLOAT, .handler = int2float },
	{ .name = "float2int",	.num = VM_FLOAT2INT, .handler = float2int },
	{ .name = "str2float",	.num = VM_STR2FLOAT, .handler = str2float },
	{ .name = "str2int",	.num = VM_STR2INT,   .handler = str2int },
	{ .name = NULL,		.num = 0,	     .handler = NULL }
};

void vm_libcall_init()
{
	unsigned int i;
	ret_t ret;

	for (i = 0; libcalls[i].name != NULL; i++) {
		ret = vm_libcall_register(&libcalls[i]);
		if (ret != ret_ok)
			xerror(1, "can't register `%s' library call",
					libcalls[i].name);
	}
}

